//
// Copyright (C) Microsoft. All rights reserved.
//
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("request");
const path = require("path");
const fs = require("fs");
const os = require("os");
const which = require("which");
const child_process_promise_1 = require("child-process-promise");
const logger_1 = require("../logger");
const adapter_1 = require("./adapter");
const adapterCollection_1 = require("./adapterCollection");
const ios8_1 = require("../protocols/ios/ios8");
const ios9_1 = require("../protocols/ios/ios9");
class IOSAdapter extends adapterCollection_1.AdapterCollection {
    constructor(id, socket, proxySettings) {
        super(id, socket, {
            port: proxySettings.proxyPort,
            proxyExePath: proxySettings.proxyPath,
            proxyExeArgs: proxySettings.proxyArgs
        });
        this._proxySettings = proxySettings;
        this._protocolMap = new Map();
    }
    getTargets() {
        logger_1.debug(`iOSAdapter.getTargets`);
        logger_1.Logger.log('iosAdapter.getTargets');
        return new Promise((resolve) => {
            request(this._url, (error, response, body) => {
                if (error) {
                    resolve([]);
                    return;
                }
                const devices = JSON.parse(body);
                resolve(devices);
            });
        }).then((devices) => {
            // Now request the device version for each device found
            const deviceVersions = [];
            devices.forEach(d => {
                let getter;
                if (d.deviceId === 'SIMULATOR') {
                    d.version = '9.3.0'; // TODO: Find a way to auto detect version. Currently hardcoding it.
                    getter = Promise.resolve(d);
                }
                else {
                    getter = this.getDeviceVersion(d.deviceId).then(v => {
                        d.version = v;
                        return Promise.resolve(d);
                    }).catch((err) => {
                        // Device version detection failed, using fallback
                        logger_1.Logger.log(`error.iosAdapter.getTargets.getDeviceVersion.failed.fallback, device=${d}`);
                        d.version = 'NaN';
                        return Promise.resolve(d);
                    });
                }
                deviceVersions.push(getter);
            });
            return Promise.all(deviceVersions);
        }).then((devices) => {
            // Now start up all the adapters
            devices.forEach(d => {
                const adapterId = `${this._id}_${d.deviceId}`;
                if (!this._adapters.has(adapterId)) {
                    const parts = d.url.split(':');
                    if (parts.length > 1) {
                        // Get the port that the ios proxy exe is forwarding for this device
                        const port = parseInt(parts[1], 10);
                        // Create a new adapter for this device and add it to our list
                        const adapter = new adapter_1.Adapter(adapterId, this._proxyUrl, { port: port });
                        adapter.start();
                        adapter.on('socketClosed', (id) => {
                            this.emit('socketClosed', id);
                        });
                        this._adapters.set(adapterId, adapter);
                    }
                }
            });
            return Promise.resolve(devices);
        }).then((devices) => {
            // Now get the targets for each device adapter in our list
            return super.getTargets(devices);
        });
    }
    connectTo(url, wsFrom) {
        const target = super.connectTo(url, wsFrom);
        if (!target) {
            throw new Error(`Target not found for ${url}`);
        }
        if (!this._protocolMap.has(target)) {
            const version = target.data.metadata.version;
            const protocol = this.getProtocolFor(version, target);
            this._protocolMap.set(target, protocol);
        }
        return target;
    }
    static getProxySettings(args) {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.debug(`iOSAdapter.getProxySettings`);
            let settings = null;
            // Check that the proxy exists
            const proxyPath = yield IOSAdapter.getProxyPath();
            // Start with remote debugging enabled
            // Use default parameters for the ios_webkit_debug_proxy executable
            const proxyPort = args.proxyPort;
            const proxyArgs = [
                '--no-frontend',
                '--config=null:' + proxyPort + ',:' + (proxyPort + 1) + '-' + (proxyPort + 101)
            ];
            settings = {
                proxyPath: proxyPath,
                proxyPort: proxyPort,
                proxyArgs: proxyArgs
            };
            return settings;
        });
    }
    static getProxyPath() {
        logger_1.debug(`iOSAdapter.getProxyPath`);
        return new Promise((resolve, reject) => {
            if (os.platform() === 'win32') {
                const x64 = os.arch() === 'x64';
                const iosVersion = process.env.ios;
                // const proxy = x64 ? path.resolve(__dirname, process.env.USERPROFILE + '/scoop/apps/ios-webkit-debug-proxy/current/ios_webkit_debug_proxy.exe') : path.resolve(__dirname, process.env.USERPROFILE + '/AppData/Roaming/npm/node_modules/vs-libimobile/lib/ios_webkit_debug_proxy.exe');
                const proxy = path.resolve(__dirname, iosVersion === '8' ? '../../node_modules/vs-libimobile/lib/ios_webkit_debug_proxy.exe' : '../../plugin/ios11/ios_webkit_debug_proxy.exe');
                try {
                    fs.statSync(proxy);
                    resolve(proxy);
                }
                catch (err) {
                    let message = x64 ? `ios_webkit_debug_proxy.exe not found. Please install 'scoop install ios-webkit-debug-proxy'` : `ios_webkit_debug_proxy.exe not found. Please install 'npm install -g vs-libimobile'`;
                    reject(message);
                }
            }
            else if (os.platform() === 'darwin' || os.platform() === 'linux') {
                which('ios_webkit_debug_proxy', function (err, resolvedPath) {
                    if (err) {
                        reject('ios_webkit_debug_proxy not found. Please install ios_webkit_debug_proxy (https://github.com/google/ios-webkit-debug-proxy)');
                    }
                    else {
                        resolve(resolvedPath);
                    }
                });
            }
        });
    }
    static getDeviceInfoPath() {
        logger_1.debug(`iOSAdapter.getDeviceInfoPath`);
        return new Promise((resolve, reject) => {
            if (os.platform() === 'win32') {
                const iosVersion = process.env.ios;
                // const proxy = path.resolve(__dirname, process.env.USERPROFILE + '/AppData/Roaming/npm/node_modules/vs-libimobile/lib/ideviceinfo.exe');
                const proxy = path.resolve(__dirname, iosVersion === '8' ? '../../node_modules/vs-libimobile/lib/ideviceinfo.exe' : '../../plugin/ios11/ideviceinfo.exe');
                try {
                    fs.statSync(proxy);
                    resolve(proxy);
                }
                catch (e) {
                    reject(`ideviceinfo not found. Please install 'npm install -g vs-libimobile'`);
                }
            }
            else if (os.platform() === 'darwin' || os.platform() === 'linux') {
                which('ideviceinfo', function (err, resolvedPath) {
                    if (err) {
                        reject('ideviceinfo not found. Please install libimobiledevice (https://github.com/libimobiledevice/libimobiledevice)');
                    }
                    else {
                        resolve(resolvedPath);
                    }
                });
            }
        });
    }
    getDeviceVersion(uuid) {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.debug(`iOSAdapter.getDeviceVersion`);
            const _iDeviceInfoPath = yield IOSAdapter.getDeviceInfoPath();
            var proc = yield child_process_promise_1.execFile(_iDeviceInfoPath, ['-u', `${uuid}`, '-k', 'ProductVersion']);
            let deviceVersion = '';
            if (!proc.err) {
                deviceVersion = proc.stdout.trim();
            }
            return deviceVersion;
        });
    }
    ;
    getProtocolFor(version, target) {
        logger_1.debug(`iOSAdapter.getProtocolFor`);
        const parts = version.split('.');
        if (parts.length > 0) {
            const major = parseInt(parts[0], 10);
            if (major <= 8) {
                return new ios8_1.IOS8Protocol(target);
            }
        }
        return new ios9_1.IOS9Protocol(target);
    }
}
exports.IOSAdapter = IOSAdapter;
